@@ -172,6 +172,131 @@ class ExportDeanonymizationTable(luigi.Task):
         # fmt: on
 
 
+class DeanonymizeContributors(luigi.Task):
+    """Generates a .csv.zst file that contains the columns:
+    ``contributor_id``, ``contributor_base64``, and ``contributor_escaped``.
+
+    The first column is the id of the contributor in the compressed graph, and
+    the latter two are the same as in the file produced by
+    :class:`ExportDeanonymizationTable`."""
+
+    local_graph_path = luigi.PathParameter()
+    graph_name = luigi.Parameter(default="graph")
+    deanonymization_table_path = luigi.PathParameter()
+    mph_algo = luigi.ChoiceParameter(choices=["cmph", "pthash"], default="pthash")
+    deanonymization_mapping_path = luigi.PathParameter()
+
+    def requires(self) -> List[luigi.Task]:
+        """Returns an instance of :class:`ExportDeanonymizationTable`."""
+        return [
+            ExportDeanonymizationTable(
+                deanonymization_table_path=self.deanonymization_table_path
+            )
+        ]
+
+    def output(self) -> luigi.Target:
+        """.csv.zst file that contains the contributor_id<->contributor_name
+        map, as well as the escaped version of the contributor name."""
+        return luigi.LocalTarget(self.deanonymization_mapping_path)
+
+    def run(self) -> None:
+        """Loads the list of persons (``graph.persons.csv.zst`` in the graph dataset
+        and the deanonymization table in memory, then uses them to map each row
+        in the original (anonymized) contributors list to the deanonymized one."""
+        import base64
+        import csv
+
+        import pyzstd
+        import tqdm
+
+        from swh.graph.shell import AtomicFileSink, Command, Rust, Sink
+
+        # Load the deanonymization table, to map sha256(name) to base64(name)
+        # and escape(name)
+        sha256_to_names: Dict[bytes, Tuple[str, str]] = {}
+        with pyzstd.open(self.deanonymization_table_path, "rt") as fd:
+            # TODO: remove that cast once we dropped Python 3.7 support
+            csv_reader = csv.reader(cast(Iterable[str], fd))
+            header = next(csv_reader)
+            assert header == ["sha256_base64", "base64", "escaped"], header
+            for csv_line in tqdm.tqdm(
+                csv_reader, unit_scale=True, desc="Loading deanonymization table"
+            ):
+                (base64_sha256_name, base64_name, escaped_name) = csv_line
+                sha256_name = base64.b64decode(base64_sha256_name)
+                sha256_to_names[sha256_name] = (base64_name, escaped_name)
+
+        # Combine with the list of sha256(name), to get the list of base64(name)
+        # and escape(name)
+        logger.info("Computing person ids using MPH...")
+        persons_path = self.local_graph_path / f"{self.graph_name}.persons.csv.zst"
+        if persons_path.exists():
+            # It can exist for some graphs (e.g. 2024-08-23)
+            persons_paths = [persons_path]
+        else:
+            # some other graphs
+            persons_dir = self.local_graph_path / f"{self.graph_name}.persons"
+            persons_paths = list(persons_dir.glob("*.zst"))
+            # Newer graphs don't have that file anymore, so we need to recreate it
+            if not persons_paths:
+                (
+                    Rust(
+                        "swh-graph-extract",
+                        "extract-persons",
+                        self.local_graph_path.parent / "orc",
+                    )
+                    | Command.zstdmt()
+                    > AtomicFileSink(persons_path)
+                ).run()
+            persons_paths = [persons_path]
+        # fmt: off
+        person_ids = (
+            Command.pv(*persons_paths)
+            | Command.zstdcat()
+            | Rust(
+                "swh-graph-hash",
+                "persons",
+                "--mph-algo",
+                self.mph_algo,
+                "--mph",
+                self.local_graph_path / f"{self.graph_name}.persons.{self.mph_algo}",
+                *(["--workaround-2024-08-23"] if self.graph_name == "2024-08-23" else []),
+            )
+            > Sink()
+        ).run()
+        nb_persons = person_ids.stdout.count(b"\n")
+        person_ids_it = iter(person_ids.stdout.decode("ascii").split("\n"))
+        # fmt: on
+
+        person_id_to_names: Dict[int, Tuple[bytes, str]] = {}
+        tmp_mapping_path = Path(f"{self.deanonymization_mapping_path.parent}.tmp")
+        tmp_mapping_path.parent.mkdir(parents=True, exist_ok=True)
+        for persons_path in persons_paths:
+            with pyzstd.open(persons_path, "rt") as fd:
+                with pyzstd.open(tmp_mapping_path, "wt") as output_fd:
+                    csv_writer = csv.writer(output_fd, lineterminator="\r\n")
+                    # write header
+                    csv_writer.writerow(("id", "base64", "escaped"))
+                    for line in tqdm.tqdm(
+                        fd,
+                        unit_scale=True,
+                        total=nb_persons - len(person_id_to_names),
+                        desc="Getting person ids",
+                    ):
+                        base64_name, name = sha256_to_names.pop(
+                            base64.b64decode(line.strip()), ("", "")
+                        )
+                        csv_writer.writerow(
+                            (int(next(person_ids_it)), base64_name, name)
+                        )
+
+        assert (
+            next(person_ids_it) == ""
+        ), "swh-graph-hash output has fewer lines than its input"
+
+        tmp_mapping_path.replace(self.deanonymization_mapping_path)
+
+
 class DeanonymizeOriginContributors(luigi.Task):
     """Generates a .csv.zst file similar to :class:`ListOriginContributors`'s,
     but with ``contributor_base64`` and ``contributor_escaped`` columns in addition to
@@ -188,10 +313,11 @@ class DeanonymizeOriginContributors(luigi.Task):
     deanonymization_table_path = luigi.PathParameter()
     deanonymized_origin_contributors_path = luigi.PathParameter()
     mph_algo = luigi.ChoiceParameter(choices=["cmph", "pthash"], default="pthash")
+    deanonymization_mapping_path = luigi.PathParameter()
 
     def requires(self) -> List[luigi.Task]:
         """Returns instances of :class:`LocalGraph`, :class:`ListOriginContributors`,
-        and :class:`ExportDeanonymizationTable`."""
+        :class:`ExportDeanonymizationTable` and :class:`DeanonymizeContributors`."""
         return [
             LocalGraph(local_graph_path=self.local_graph_path),
             ListOriginContributors(
@@ -201,6 +327,13 @@ class DeanonymizeOriginContributors(luigi.Task):
             ExportDeanonymizationTable(
                 deanonymization_table_path=self.deanonymization_table_path,
             ),
+            DeanonymizeContributors(
+                local_graph_path=self.local_graph_path,
+                graph_name=self.graph_name,
+                deanonymization_table_path=self.deanonymization_table_path,
+                mph_algo=self.mph_algo,
+                deanonymization_mapping_path=self.deanonymization_mapping_path,
+            ),
         ]
 
     def output(self) -> luigi.Target:
@@ -222,71 +355,6 @@ class DeanonymizeOriginContributors(luigi.Task):
         import pyzstd
         import tqdm
 
-        from swh.graph.shell import Command, Rust, Sink
-
-        # Load the deanonymization table, to map sha256(name) to base64(name)
-        # and escape(name)
-        sha256_to_names: Dict[bytes, Tuple[bytes, str]] = {}
-        with pyzstd.open(self.deanonymization_table_path, "rt") as fd:
-            # TODO: remove that cast once we dropped Python 3.7 support
-            csv_reader = csv.reader(cast(Iterable[str], fd))
-            header = next(csv_reader)
-            assert header == ["sha256_base64", "base64", "escaped"], header
-            for csv_line in tqdm.tqdm(
-                csv_reader, unit_scale=True, desc="Loading deanonymization table"
-            ):
-                (base64_sha256_name, base64_name, escaped_name) = csv_line
-                sha256_name = base64.b64decode(base64_sha256_name)
-                name = base64.b64decode(base64_name)
-                sha256_to_names[sha256_name] = (name, escaped_name)
-
-        # Combine with the list of sha256(name), to get the list of base64(name)
-        # and escape(name)
-        logger.info("Computing person ids using MPH...")
-        persons_path = self.local_graph_path / f"{self.graph_name}.persons.csv.zst"
-        if persons_path.exists():
-            # 2024-08-23 graph
-            persons_paths = [persons_path]
-        else:
-            # other graphs (before and after)
-            persons_dir = self.local_graph_path / f"{self.graph_name}.persons"
-            persons_paths = list(persons_dir.glob("*.zst"))
-            assert persons_paths, f"{persons_dir} is empty"
-        # fmt: off
-        person_ids = (
-            Command.pv(*persons_paths)
-            | Command.zstdcat()
-            | Rust(
-                "swh-graph-hash",
-                "persons",
-                "--mph-algo",
-                self.mph_algo,
-                "--mph",
-                self.local_graph_path / f"{self.graph_name}.persons.{self.mph_algo}",
-                *(["--workaround-2024-08-23"] if self.graph_name == "2024-08-23" else []),
-            )
-            > Sink()
-        ).run()
-        nb_persons = person_ids.stdout.count(b"\n")
-        person_ids_it = iter(person_ids.stdout.decode("ascii").split("\n"))
-        # fmt: on
-        person_id_to_names: Dict[int, Tuple[bytes, str]] = {}
-        for persons_path in persons_paths:
-            with pyzstd.open(persons_path, "rb") as fd:
-                for line in tqdm.tqdm(
-                    fd,
-                    unit_scale=True,
-                    total=nb_persons - len(person_id_to_names),
-                    desc="Getting person ids",
-                ):
-                    person_id_to_names[int(next(person_ids_it))] = sha256_to_names.pop(
-                        base64.b64decode(line.strip()), (b"", "")
-                    )
-
-        assert (
-            next(person_ids_it) == ""
-        ), "swh-graph-hash output has fewer lines than its input"
-
         # Read the set of person ids from the main table
         person_ids = set()
         with pyzstd.open(self.origin_contributors_path, "rt") as input_fd:
@@ -305,6 +373,17 @@ class DeanonymizeOriginContributors(luigi.Task):
                     continue
                 person_ids.add(int(person_id_str))
 
+        person_id_to_names: Dict[int, Tuple[bytes, str]] = {}
+        with pyzstd.open(self.deanonymization_mapping_path, "rt") as mapping_fd:
+            # TODO: remove that cast once we dropped Python 3.7 support
+            csv_reader = csv.reader(cast(Iterable[str], mapping_fd))
+            header = next(csv_reader)
+            assert header == ["id", "base64", "escaped"], header
+            for id, base64_name, escaped in tqdm.tqdm(
+                csv_reader, unit_scale=True, desc="Reading mapping from id to name"
+            ):
+                person_id_to_names[int(id)] = (base64.b64decode(base64_name), escaped)
+
         # Finally, write a new table of all persons.
         tmp_output_path = Path(f"{self.deanonymized_origin_contributors_path}.tmp")
         tmp_output_path.parent.mkdir(parents=True, exist_ok=True)
